function crm00001000_Parametrizacao_Crm_Controller(General, CRM_Utilities, templateManager, ScreenService, restEngine, Messages, IntegracaoRDStationController) {
    
    /**
     * Procedimento para manter uma parametrização.
     * 
     * @param {object} widget widget referente a tela
     * 'parametrização CRM
    */
    this.saveParameterization = function(widget){
        if(!this.checkIfThereIsRecord(widget)){
            this.saveNewParameterization(widget);
        }else{
            this.editParameterization(widget);
        }
    };
    
    /**
     * Procedimento para salvar uma nova parametrização.
     * 
     * @param {object} widget widget referente a tela
     * 'parametrização CRM'.
     * 
     * Alterado 22/01/18 - Manda pro backend se será
     * necessário criptografar a senha do IMAP
    */
    this.saveNewParameterization = function(widget){
        
        var currentRow = widget.currentRow;
        if(widget.isValid()) {
            if(Util.isEmptyOrBlank(currentRow.NRTPARVPROD)) {
                currentRow.NRTPARVPROD = null;
            }
            
            this.defineCurrentRowAsNewRecord(widget);
            this.setNrDiasLancEvolInTemplateManager(currentRow.NRDIASLANCEVOL);
            this.controlMultipleTree(widget, 'save');
            CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRSEQPARAMETRIZACAO");
            
            widget.currentRow.TXHOSTIMAP = (typeof widget.currentRow.TXHOSTIMAP === "string" && widget.currentRow.TXHOSTIMAP !== '') ? widget.currentRow.TXHOSTIMAP.split(':')[0] + ':' + widget.currentRow.HOSTPORT : ''; 
            
            var redirectRDStation = false;
            if(widget.name == 'WDGT_PARAM_INTEGRATION_RDSTATION'){
                this.prepareTags(widget);
                IntegracaoRDStationController.showCustomFieldAction();
                redirectRDStation = true;
            }
                
            General.widgetSave(widget).then(function(response){
                currentRow.NRSEQPARAMETRIZACAO = response.dataset.columnsToReturn.NRSEQPARAMETRIZACAO;
                var widgetGeral = templateManager.container.getWidget('WDGT_PARAM_CRM');
                widgetGeral.reload().then(function(){
                    templateManager.CRM_PARAMETRIZACAO = widget.dataSource.data[0];
                    widget.currentRow.OLDPASSIMAP = widget.dataSource.data[0].TXPASSIMAP;    
                    widget.currentRow.TXHOSTIMAP = (typeof widget.currentRow.TXHOSTIMAP === "string" && widget.widgetGeral.TXHOSTIMAP !== '') ? widget.widgetGeral.TXHOSTIMAP.split(':')[0] : '';
                    General.changeWidgetAcessibility(templateManager.container.getWidget('WDGT_TABBED_PARAM_CRM').widgets, true);
                    if(redirectRDStation)
                        IntegracaoRDStationController.authRDStation(widget);
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Procedimento para editar a parametrização
     * existente.
     * 
     * @param {object} widget widget referente a tela
     * 'parametrização CRM'.
     * 
     * Alterado 22/01/18 - Manda pro backend se será
     * necessário criptografar a senha do IMAP
    */
    this.editParameterization = function(widget){
        if(widget.isValid()){
            if(Util.isEmptyOrBlank(widget.currentRow.NRTPARVPROD)) {
                widget.currentRow.NRTPARVPROD = null;
            }
            
            this.setNrDiasLancEvolInTemplateManager(widget.currentRow.NRDIASLANCEVOL);
            this.controlMultipleTree(widget, 'save');
            
            widget.currentRow.TXHOSTIMAP = (typeof widget.currentRow.TXHOSTIMAP === "string" && widget.currentRow.TXHOSTIMAP !== '') ? widget.currentRow.TXHOSTIMAP.split(':')[0] + ':' + widget.currentRow.HOSTPORT : '';  
            
            var redirectRDStation = false;
            if(widget.name == 'WDGT_PARAM_INTEGRATION_RDSTATION'){
                this.prepareTags(widget);
                IntegracaoRDStationController.showCustomFieldAction();
                redirectRDStation = true;
            }
                
            General.widgetSave(widget).then(function(){
                var widgetGeral = templateManager.container.getWidget('WDGT_PARAM_CRM');
                widgetGeral.reload().then(function(){
                    templateManager.CRM_PARAMETRIZACAO = widget.dataSource.data[0];
                    widget.currentRow.OLDPASSIMAP = widgetGeral.dataSource.data[0].TXPASSIMAP;    
                    widget.currentRow.TXHOSTIMAP = (typeof widgetGeral.currentRow.TXHOSTIMAP === "string" && widgetGeral.currentRow.TXHOSTIMAP !== '') ? widgetGeral.currentRow.TXHOSTIMAP.split(':')[0] : '';
                    General.changeWidgetAcessibility(templateManager.container.getWidget('WDGT_TABBED_PARAM_CRM').widgets, true);
                    if(redirectRDStation)
                        IntegracaoRDStationController.authRDStation(widget);
                }.bind(this));
            }.bind(this));
        }
    };
    
    /**
     * Procedimento para verificar se existe
     * algum registro na tabela referente ao
     * dataSource da tela de parametrização.
     * 
     * @param {object} widget widget referente a tela
     * 'parametrização CRM
     * 
     * @return {boolean} retorna um valor lógico
     * mostrando se exite algum registro.
    */
    this.checkIfThereIsRecord = function(widget){
        var records = widget.dataSource.data;
        
        return records.length > 0;
    };
    
    /**
     * Procedimento para definir a linha atual
     * como novo registro, definindo a variável
     * __is_new como 'true'.
     * 
     * @param {object} widget widget referente
     * a currentRow em caso.
    */
    this.defineCurrentRowAsNewRecord = function(widget){
        widget.currentRow.__is_new = true;
    };
    
    /**
     * Procedimento para setar as informações da 
     * parametrização existente na current row da
     * widget de parametrização em modo de visualização.
     * 
     * @param {object} widget widget referente a tela
     * 'parametrização CRM
    */
    this.setParameterizationInCurrentRow = function(widget){
        if(this.checkIfThereIsRecord(widget)){
            widget.currentRow = widget.dataSource.data[0];
            if(!Util.isEmptyOrBlank(widget.currentRow.TXPASSIMAP)){
                widget.currentRow.HOST = widget.currentRow.TXHOSTIMAP;
                widget.currentRow.OLDPASSIMAP = widget.currentRow.TXPASSIMAP;
                widget.currentRow.HOSTPORT =   widget.currentRow.HOST.split(':')[1]; 
                widget.currentRow.TXHOSTIMAP = widget.currentRow.HOST.split(':')[0]; 
            }
            
            if(widget.name == 'WDGT_PARAM_CRM') {
                widget.getField('NRTPARVPROD').reload();
            }
        }else{
            /**
             * Inicializa row com todos os campos vazios
             * para evitar erros no backend na hora de 
             * salvar.
             * 
            */
            
            widget.currentRow.NRDIASLANCEVOL = "";
            widget.currentRow.NRLIMITEPROSPOPER = "";
            widget.currentRow.NRMAXDIASPROSPBLOQ = "";
            widget.currentRow.NRPERFILACESSOWEBMAIL = "";
            widget.currentRow.NRLIMITELEADREVENDA = "";
            widget.currentRow.IDBLOQNOMESEMELHA = "";
            widget.currentRow.IDCNPJRAIZ = "";
            widget.currentRow.IDPERMITESINCGOOGLE = "";
            widget.currentRow.IDVALIDARESPOPORTUNI = "N";
            widget.currentRow.NRSEQPERFATIV = "";
            widget.currentRow.NRTPARVPROD = "";
            widget.currentRow.TXHOSTIMAP = "";
            widget.currentRow.HOSTPORT = "";
            widget.currentRow.TXUSERIMAP = "";
            widget.currentRow.TXPASSIMAP = "";
            widget.currentRow.NRPERFILACESSO = "";
            widget.currentRow.IDINTEGRACAOSM = "N";
            
            widget.currentRow.NMCONTARD = "";
            widget.currentRow.IDCONTARD = "";
            widget.currentRow.IDCLIENTRD = "";
            widget.currentRow.IDCLIENTSECRETRD = "";
            widget.currentRow.NMEVENTOSCONVERSAO = "";
            widget.currentRow.IDGERATAREFARD = "N";
            widget.currentRow.CDOPERRESPONSAVELRD = "";
            widget.currentRow.DSDOMINIOSRD = "";
            widget.currentRow.DSCARGOSRD = "";
            widget.currentRow.DSSCORERD = "";
            widget.currentRow.DSTOKENPUBLICORD = "";
            widget.currentRow.DSTOKENPRIVADORD = "";
            widget.currentRow.IDADDPROSPECT = "N";
            widget.currentRow.IDATIVO = "S";

        }  
        
        //var widgetIMAP = templateManager.container.getWidget('WDGT_PARAM_INTEGRATION_IMAP');
        var widgetRD = templateManager.container.getWidget('WDGT_PARAM_INTEGRATION_RDSTATION');
        var widgetMautic = templateManager.container.getWidget('WDGT_PARAM_INTEGRATION_MAUTIC');
        
        //widgetIMAP.currentRow = widget.currentRow;
        widgetRD.currentRow = widget.currentRow;
        widgetMautic.currentRow = widget.currentRow;
        
        //widgetIMAP.dataSource.data = widget.dataSource.data;
        widgetRD.dataSource.data = widget.dataSource.data;
        widgetMautic.dataSource.data = widget.dataSource.data;
        
        this.changeReadOnlyPerfilAtiv(widget);
    };
    
    /**
     * Procedimento para setar o campo 'NRDIASLANCEVOL'
     * no templateManager;
     * 
     * @param {number} nrDiasLanceEvol campo 'NRDIASLANCEVOL'.
     * 
    */
    this.setNrDiasLancEvolInTemplateManager = function(nrDiasLanceEvol){
        templateManager.NRDIASLANCEVOL = nrDiasLanceEvol;
    };
    
    /**
     * Procedimento para mudar o readOnly do Campo
     * 'Notificar por perfil de atividade' baseado no
     * que foi selecionado no campo 'Notificar responsável'.
     * 
     * @param {object} widget
     * 
    */
    this.changeReadOnlyPerfilAtiv = function(widget){
        var fieldNotifyOperator   = widget.getField('IDVALIDARESPOPORTUNI');
        var fieldNotifyPerfilAtiv = widget.getField('DESCPEFATIV');
        var hostField = widget.getField('DSHOSTSMTP');
        
        if(fieldNotifyOperator.value() === "N") {
            if(!Util.isEmptyOrBlank(hostField.value())) {
                fieldNotifyPerfilAtiv.readOnly = false;
            } else {
                fieldNotifyPerfilAtiv.readOnly = true;
            }
        } else {
            fieldNotifyPerfilAtiv.readOnly = true;
            fieldNotifyPerfilAtiv.clearValue();
            widget.currentRow.NRSEQPERFATIV = '';
        }
        
        if(!Util.isEmptyOrBlank(hostField.value())) {
            fieldNotifyOperator.readOnly = false;
        } else {
            fieldNotifyOperator.value("N");
            fieldNotifyOperator.readOnly = true;
        }
    };
    
    this.controlMultipleTree = function(widget, mode){
        if(!Util.isEmptyOrBlank(widget.currentRow.NRTPARVPROD)){
            if(mode === 'save')
                widget.currentRow.NRTPARVPROD = widget.currentRow.NRTPARVPROD.join(',');     
            else if(mode === 'edit')
                widget.currentRow.NRTPARVPROD = widget.currentRow.NRTPARVPROD.split(',');
        }
    };
    
    this.changeReadOnlyImap = function(widget){
        var currentRow = widget.currentRow;
        
        if(!Util.isEmptyOrBlank(currentRow.TXHOSTIMAP)){
            widget.getField('TXUSERIMAP').readOnly = false;
            widget.getField('TXUSERIMAP').validations = {required:true};
            widget.getField('TXPASSIMAP').readOnly = false;
            widget.getField('TXPASSIMAP').validations = {required:true};
            widget.getField('HOSTPORT').readOnly = false;
            widget.getField('HOSTPORT').validations = {required:true};
            $('#HOSTPORT').focus();
            widget.getField('NMPERFILACESSO').readOnly = false;
            widget.getField('NMPERFILACESSO').validations = {required:true};
        }else{
            widget.getField('TXUSERIMAP').readOnly = true;
            widget.currentRow.TXUSERIMAP = '';
            widget.getField('TXUSERIMAP').validations = {};
            widget.getField('TXPASSIMAP').readOnly = true;
            widget.currentRow.TXPASSIMAP = '';
            widget.getField('TXPASSIMAP').validations = {};
            widget.getField('HOSTPORT').readOnly = true;
            widget.currentRow.HOSTPORT = '';
            widget.getField('HOSTPORT').validations = {};
            widget.currentRow.NMPERFILACESSO = '';
            widget.currentRow.NRPERFILACESSOWEBMAIL = '';
            widget.getField('NMPERFILACESSO').readOnly = true;
            widget.getField('NMPERFILACESSO').validations = {};
        }
        
    };
    
    this.setHostInCurrentRow = function(widget){
        if(!Util.isEmptyOrBlank(widget.currentRow.HOST)){
            widget.currentRow.TXHOSTIMAP =   widget.currentRow.HOST.split(':')[0]; 
            widget.currentRow.OLDPASSIMAP = widget.currentRow.TXPASSIMAP;
        }else{
            widget.currentRow.HOST = widget.currentRow.TXHOSTIMAP;
            widget.currentRow.TXHOSTIMAP =   widget.currentRow.TXHOSTIMAP.split(':')[0]; 
        } 
    };
    
    this.prepareTags = function(widget){
        widget.fields.forEach(function(field){
            if(field.template.indexOf('field-tag') !== -1){
                widget.currentRow[field.name] = Array.isArray(widget.currentRow[field.name]) ? widget.currentRow[field.name].join(',') : '';
            }
        });
    };
    
    this.editMautic = function(widget) {
        var mainWid = templateManager.container.getWidget('WDGT_TABBED_PARAM_CRM');
        
        General.widgetEdit(widget);
        General.changeWidgetAcessibility(mainWid.widgets, true);
    };
    
    this.saveMautic = async function(widget) {
        if(!widget.isValid()) {
            return;
        }
        
        var restParams = {
            requestType: 'Row',
            row: widget.currentRow
        };
        
        await CRM_Utilities.post('/saveParamMautic', restParams);
        
        General.widgetCancelForm(widget);    
        await this.reloadAndCopyDataSource();
        ScreenService.showMessage('Integração salva com sucesso!');
        
        
    };
    
    this.cancelMautic = function(widget) {
        General.widgetCancelForm(widget);
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm00001000_Parametrizacao_Crm_Controller', crm00001000_Parametrizacao_Crm_Controller);
});